"""Automatically load a configuration according to the RUN_ENV parameter."""
import os
import logging
from importlib.util import spec_from_file_location, module_from_spec

log = logging.getLogger(__name__)


def merge_configs(a, b):
    """Recursively merge config dictionaries."""
    new_config = dict()
    for key in a.keys() ^ b.keys():
        new_config[key] = a.get(key) or b[key]

    for key in a.keys() & b.keys():
        if isinstance(a[key], dict) and isinstance(b[key], dict):
            new_config[key] = merge_configs(a[key], b[key])
        else:
            log.warn(
                'Overriding keys with different types %s and %s',
                type(a[key]), type(b[key]))
            new_config[key] = b[key]
    return new_config


def load_config(name):
    """Load config with given name.

    Looks in the config folder of the current working directory for
    a file named %{name}.py.
    """
    try:
        path = os.path.join(os.getcwd(), 'config', name.lower() + '.py')
        spec = spec_from_file_location(name, path)
        this_config = module_from_spec(spec)
        spec.loader.exec_module(this_config)
        return this_config.config
    except FileNotFoundError:
        log.warn('Not found %s config', name)
        return dict()


config = merge_configs(
    load_config('default'),
    load_config(os.environ.get('RUN_ENV') or 'development'))



import unittest


class TestConfig(unittest.TestCase):

    def test_config(self):
        from src import config
        assert(config == dict())

    def test_config_merge(self):
        one = {
            'only_in_1': True,
            'nested': {
                'only_in_1': True,
                'conflicting': False
            },
            'conflicting': False
        }
        two = {
            'only_in_2': True,
            'nested': {
                'only_in_2': True,
                'conflicting': True
            },
            'conflicting': True
        }
        from src import merge_configs
        merged = merge_configs(one, two)
        assert(merged['only_in_1'])
        assert(merged['only_in_2'])
        assert(merged['conflicting'])
        assert(merged['nested']['only_in_1'])
        assert(merged['nested']['only_in_2'])
        assert(merged['nested']['conflicting'])

    def test_bad_config_merge(self):
        one = {
            'nested': 'string'
        }
        two = {
            'nested': {
                'only_in_2': True,
                'conflicting': True
            }
        }
        from src import merge_configs
        merged = merge_configs(one, two)
        assert(merged['nested']['only_in_2'])
        assert(merged['nested']['conflicting'])

    def test_load_config(self):
        from src import load_config
        example_config = load_config('example_config')
        assert(example_config['default_string'] == 'string')



from setuptools import setup

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError, OSError):
    print('WARN: README conversion to rst skipped.')
    long_description = open('README.md').read()

setup(
    name='sagaconfig',
    packages=['sagaconfig'],
    package_dir={'sagaconfig': 'src'},
    version='0.0.1',
    description='A config helper',
    long_description=long_description,
    author='Sagacify',
    author_email='dev@sagacify.com',
    url='https://github.com/Sagacify/config-py',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5'
    ]
)
